@@ -4,10 +4,10 @@ functions and object for managing OSM maps
 #!/usr/bin/python
 
 # import official python packages
+from datetime import datetime
 import glob
 import multiprocessing
 import os
-import struct
 import subprocess
 import sys
 import platform
@@ -25,6 +25,7 @@ from wahoomc.constants import USER_OUTPUT_DIR
 from wahoomc.constants import RESOURCES_DIR
 from wahoomc.constants import LAND_POLYGONS_PATH
 from wahoomc.constants import VERSION
+from wahoomc.constants import OSMOSIS_WIN_FILE_PATH
 
 from wahoomc.downloader import Downloader
 from wahoomc.geofabrik import Geofabrik
@@ -103,6 +104,15 @@ def run_subprocess_and_log_output(cmd, error_message, cwd=""):
         sys.exit()
 
 
+def get_timestamp_last_changed(file_path):
+    """
+    returns the timestamp of the last-changed datetime of the given file
+    """
+    chg_time = os.path.getmtime(file_path)
+
+    return datetime.fromtimestamp(chg_time).isoformat()
+
+
 class OsmData():  # pylint: disable=too-few-public-methods
     """
     object with all internal parameters to process maps
@@ -291,20 +301,12 @@ class OsmMaps:
     This is a OSM data class
     """
 
-    osmosis_win_file_path = get_tooling_win_path(
-        ['Osmosis', 'bin', 'osmosis.bat'])
-
     # Number of workers for the Osmosis read binary fast function
     workers = '1'
 
     def __init__(self, o_osm_data):
         self.o_osm_data = o_osm_data
-
-        if 8 * struct.calcsize("P") == 32:
-            self.osmconvert_path = get_tooling_win_path(['osmconvert'])
-        else:
-            self.osmconvert_path = get_tooling_win_path(
-                ['osmconvert64-0.8.8p'])
+        self.osmconvert_path = get_tooling_win_path('osmconvert')
 
         create_empty_directories(
             USER_OUTPUT_DIR, self.o_osm_data.tiles, self.o_osm_data.border_countries)
@@ -332,7 +334,8 @@ class OsmMaps:
                 out_file_o5m = os.path.join(country_dir, 'outFile.o5m')
                 # only create o5m file if not there already or force processing (no user input possible)
                 # --> speeds up processing if one only wants to test tags / POIs
-                if not os.path.isfile(out_file_o5m) or self.o_osm_data.force_processing is True:
+                if not os.path.isfile(out_file_o5m) or self.o_osm_data.force_processing is True \
+                        or self.last_changed_is_identical_to_last_run(key) is False:
                     log.info('+ Converting map of %s to o5m format', key)
                     cmd = [self.osmconvert_path]
                     cmd.extend(['-v', '--hash-memory=2500', '--complete-ways',
@@ -351,10 +354,11 @@ class OsmMaps:
                 # - force processing is set (this is also when new map files were dowwnloaded)
                 # - the defined TAGS_TO_KEEP_UNIVERSAL constants have changed are changed (user input or new release)
                 if not os.path.isfile(out_file_o5m_filtered_win) or not os.path.isfile(out_file_o5m_filtered_names_win) \
-                        or self.o_osm_data.force_processing is True or self.tags_are_identical_to_last_run(key) is False:
+                        or self.o_osm_data.force_processing is True or self.tags_are_identical_to_last_run(key) is False \
+                        or self.last_changed_is_identical_to_last_run(key) is False:
                     log.info(
                         '+ Filtering unwanted map objects out of map of %s', key)
-                    cmd = [get_tooling_win_path(['osmfilter'])]
+                    cmd = [get_tooling_win_path('osmfilter', in_user_dir=True)]
                     cmd.append(out_file_o5m)
                     cmd.append(
                         '--keep="' + translate_tags_to_keep(sys_platform=platform.system()) + '"')
@@ -365,7 +369,7 @@ class OsmMaps:
                     run_subprocess_and_log_output(
                         cmd, f'! Error in OSMFilter with country: {key}')
 
-                    cmd = [get_tooling_win_path(['osmfilter'])]
+                    cmd = [get_tooling_win_path('osmfilter', in_user_dir=True)]
                     cmd.append(out_file_o5m)
                     cmd.append(
                         '--keep="' + translate_tags_to_keep(
@@ -391,7 +395,8 @@ class OsmMaps:
                 # - force processing is set (this is also when new map files were dowwnloaded)
                 # - the defined TAGS_TO_KEEP_UNIVERSAL constants have changed are changed (user input or new release)
                 if not os.path.isfile(out_file_pbf_filtered_mac) or not os.path.isfile(out_file_pbf_filtered_names_mac) \
-                        or self.o_osm_data.force_processing is True or self.tags_are_identical_to_last_run(key) is False:
+                        or self.o_osm_data.force_processing is True or self.tags_are_identical_to_last_run(key) is False \
+                        or self.last_changed_is_identical_to_last_run(key) is False:
                     log.info(
                         '+ Filtering unwanted map objects out of map of %s', key)
 
@@ -608,7 +613,7 @@ class OsmMaps:
         tile_count = 1
         for tile in self.o_osm_data.tiles:  # pylint: disable=too-many-nested-blocks
             log.info(
-                '+ Merging tiles for tile %s of %s for Coordinates: %s,%s', tile_count, len(self.o_osm_data.tiles), tile["x"], tile["y"])
+                '+ Coordinates: %s,%s (%s of %s)', tile["x"], tile["y"], tile_count, len(self.o_osm_data.tiles))
 
             out_tile_dir = os.path.join(USER_OUTPUT_DIR,
                                         f'{tile["x"]}', f'{tile["y"]}')
@@ -622,51 +627,39 @@ class OsmMaps:
 
             # Windows
             if platform.system() == "Windows":
-                cmd = [self.osmosis_win_file_path]
-                loop = 0
-                # loop through all countries of tile, if border-countries should be processed.
-                # if border-countries should not be processed, only process the "entered" country
-                for country in tile['countries']:
-                    if process_border_countries or country in self.o_osm_data.border_countries:
-                        cmd.append('--rbf')
-                        cmd.append(os.path.join(
-                            out_tile_dir, f'split-{country}.osm.pbf'))
-                        cmd.append('workers=' + self.workers)
-                        if loop > 0:
-                            cmd.append('--merge')
-
-                        cmd.append('--rbf')
-                        cmd.append(os.path.join(
-                            out_tile_dir, f'split-{country}-names.osm.pbf'))
-                        cmd.append('workers=' + self.workers)
-                        cmd.append('--merge')
-
-                        loop += 1
-
-                for land in land_files:
-                    cmd.extend(
-                        ['--rx', 'file='+os.path.join(out_tile_dir, f'{land}'), '--s', '--m'])
-                cmd.extend(
-                    ['--rx', 'file='+os.path.join(out_tile_dir, 'sea.osm'), '--s', '--m'])
-                cmd.extend(['--tag-transform', 'file=' + os.path.join(RESOURCES_DIR,
-                                                                      'tunnel-transform.xml'), '--wb', out_file_merged, 'omitmetadata=true'])
-
+                cmd = [OSMOSIS_WIN_FILE_PATH]
             # Non-Windows
             else:
-                cmd = ['osmium', 'merge', '--overwrite']
-                # loop through all countries of tile, if border-countries should be processed.
-                # if border-countries should not be processed, only process the "entered" country
-                for country in tile['countries']:
-                    if process_border_countries or country in self.o_osm_data.border_countries:
-                        cmd.append(os.path.join(
-                            out_tile_dir, f'split-{country}.osm.pbf'))
-                        cmd.append(os.path.join(
-                            out_tile_dir, f'split-{country}-names.osm.pbf'))
+                cmd = ['osmosis']
 
-                for land in land_files:
-                    cmd.append(land)
-                cmd.append(os.path.join(out_tile_dir, 'sea.osm'))
-                cmd.extend(['-o', out_file_merged])
+            loop = 0
+            # loop through all countries of tile, if border-countries should be processed.
+            # if border-countries should not be processed, only process the "entered" country
+            for country in tile['countries']:
+                if process_border_countries or country in self.o_osm_data.border_countries:
+                    cmd.append('--rbf')
+                    cmd.append(os.path.join(
+                        out_tile_dir, f'split-{country}.osm.pbf'))
+                    cmd.append('workers=' + self.workers)
+                    if loop > 0:
+                        cmd.append('--merge')
+
+                    cmd.append('--rbf')
+                    cmd.append(os.path.join(
+                        out_tile_dir, f'split-{country}-names.osm.pbf'))
+                    cmd.append('workers=' + self.workers)
+                    cmd.append('--merge')
+
+                    loop += 1
+
+            for land in land_files:
+                cmd.extend(
+                    ['--rx', 'file='+land, '--s', '--m'])
+
+            cmd.extend(
+                ['--rx', 'file='+os.path.join(out_tile_dir, 'sea.osm'), '--s', '--m'])
+            cmd.extend(['--tag-transform', 'file=' + os.path.join(RESOURCES_DIR,
+                                                                  'tunnel-transform.xml'), '--wb', out_file_merged, 'omitmetadata=true'])
 
             run_subprocess_and_log_output(
                 cmd, f'! Error in Osmosis with tile: {tile["x"]},{tile["y"]}')
@@ -682,33 +675,27 @@ class OsmMaps:
         https://github.com/osmcode/osmium-tool/releases/tag/v1.13.2
         """
 
-        log.info('-' * 80)
-        log.info('# Sorting land* osm files')
+        log.debug('-' * 80)
+        log.debug('# Sorting land* osm files')
 
         # get all land* osm files
         land_files = glob.glob(os.path.join(USER_OUTPUT_DIR,
                                             f'{tile["x"]}', f'{tile["y"]}', 'land*.osm'))
 
-        # Windows
-        if platform.system() == "Windows":
-            for land in land_files:
-                cmd = [self.osmosis_win_file_path]
+        for land in land_files:
+            if platform.system() == "Windows":
+                cmd = [OSMOSIS_WIN_FILE_PATH]
+            else:
+                cmd = ['osmosis']
 
-                cmd.extend(['--read-xml', 'file='+os.path.join(land)])
-                cmd.append('--sort')
-                cmd.extend(['--write-xml', 'file='+os.path.join(land)])
-
-        # Non-Windows
-        else:
-            for land in land_files:
-                cmd = ['osmium', 'sort', '--overwrite']
-                cmd.append(land)
-                cmd.extend(['-o', land])
+            cmd.extend(['--read-xml', 'file='+land])
+            cmd.append('--sort')
+            cmd.extend(['--write-xml', 'file='+land])
 
         run_subprocess_and_log_output(
             cmd, f'Error in Osmosis with sorting land* osm files of tile: {tile["x"]},{tile["y"]}')
 
-        log.info('+ Sorting land* osm files: OK')
+        log.debug('+ Sorting land* osm files: OK')
 
     def create_map_files(self, save_cruiser, tag_wahoo_xml):
         """
@@ -737,7 +724,7 @@ class OsmMaps:
 
             # Windows
             if platform.system() == "Windows":
-                cmd = [self.osmosis_win_file_path, '--rbf', merged_file,
+                cmd = [OSMOSIS_WIN_FILE_PATH, '--rbf', merged_file,
                        'workers=' + self.workers, '--mw', 'file='+out_file_map]
             # Non-Windows
             else:
@@ -762,7 +749,7 @@ class OsmMaps:
 
             # Windows
             if platform.system() == "Windows":
-                cmd = [get_tooling_win_path(['lzma']), 'e', out_file_map,
+                cmd = [get_tooling_win_path('lzma'), 'e', out_file_map,
                        out_file_map+'.lzma', f'-mt{threads}', '-d27', '-fb273', '-eos']
             # Non-Windows
             else:
@@ -824,7 +811,7 @@ class OsmMaps:
         if zip_folder:
             # Windows
             if platform.system() == "Windows":
-                cmd = [get_tooling_win_path(['7za']), 'a', '-tzip']
+                cmd = [get_tooling_win_path('7za'), 'a', '-tzip']
 
                 cmd.extend(
                     [folder_name + '.zip', os.path.join(".", folder_name, "*")])
@@ -876,7 +863,9 @@ class OsmMaps:
         # Data to be written
         configuration = {
             "version_last_run": VERSION,
-            "tags_last_run": translate_tags_to_keep()
+            "changed_ts_map_last_run": get_timestamp_last_changed(self.o_osm_data.border_countries[country]['map_file']),
+            "tags_last_run": translate_tags_to_keep(sys_platform=platform.system()),
+            "name_tags_last_run": translate_tags_to_keep(name_tags=True, sys_platform=platform.system())
         }
 
         write_json_file_generic(os.path.join(
@@ -891,9 +880,26 @@ class OsmMaps:
         try:
             country_config = read_json_file(os.path.join(
                 USER_OUTPUT_DIR, country, ".config.json"))
-            if not country_config["tags_last_run"] == translate_tags_to_keep(sys_platform=platform.system()):
+            if not country_config["tags_last_run"] == translate_tags_to_keep(sys_platform=platform.system()) \
+                    or not country_config["name_tags_last_run"] == translate_tags_to_keep(name_tags=True, sys_platform=platform.system()):
                 tags_are_identical = False
         except (FileNotFoundError, KeyError):
             tags_are_identical = False
 
         return tags_are_identical
+
+    def last_changed_is_identical_to_last_run(self, country):
+        """
+        compare tags of this run with used tags from last run stored in _tiles/{country} directory
+        """
+        last_changed_is_identical = True
+
+        try:
+            country_config = read_json_file(os.path.join(
+                USER_OUTPUT_DIR, country, ".config.json"))
+            if not country_config["changed_ts_map_last_run"] == get_timestamp_last_changed(self.o_osm_data.border_countries[country]['map_file']):
+                last_changed_is_identical = False
+        except (FileNotFoundError, KeyError):
+            last_changed_is_identical = False
+
+        return last_changed_is_identical
